@@ -49,7 +49,7 @@ from . import __version__
 from . import plugin
 from . import options
 
-from .util import pushdir
+from .util import pushdir, push_python_path
 
 
 def install_plugin(p):
@@ -74,8 +74,8 @@ def load_model(path, model_options=None):
     from .fitproblem import load_problem
 
     # Change to the target path before loading model so that data files
-    # can be given as relative paths in the model file.  This should also
-    # allow imports as expected from the model file.
+    # can be given as relative paths in the model file.  Add the directory
+    # to the python path (at the end) so that imports work as expected.
     directory, filename = os.path.split(path)
     with pushdir(directory):
         # Try a specialized model loader
@@ -140,16 +140,33 @@ def load_best(problem, path):
     """
     Load parameter values from a file.
     """
-    #targets = dict(zip(problem.labels(), problem.getp()))
-    targets = dict((name, np.NaN) for name in problem.labels())
+    # Reload the individual parameters from a saved par file. Use the value
+    # from the model as the default value.  Keep track of which parameters are
+    # defined in the file so we can see if any are missing.
+    targets = dict(zip(problem.labels(), problem.getp()))
+    defined = set()
+    if not os.path.isfile(path):
+        path = os.path.join(path, problem.name+".par")
     with open(path, 'rt') as fid:
         for line in fid:
             m = PARS_PATTERN.match(line)
             label, value = m.group('label'), float(m.group('value'))
             if label in targets:
                 targets[label] = value
+                defined.add(label)
     values = [targets[label] for label in problem.labels()]
     problem.setp(np.asarray(values))
+
+    # Identify the missing parameters if any.  These are stuffed into the
+    # the problem definition as an optional "undefined" attribute, with
+    # one bit for each parameter.  If all parameters are defined, then none
+    # are undefined.  This ugly hack is to support a previous ugly hack in
+    # which undefined parameters are initialized with LHS but defined
+    # parameters are initialized with eps, cov or random.
+    # TODO: find a better way to "free" parameters on --resume/--pars.
+    if len(values) != len(defined):
+        undefined = [label not in defined for label in problem.labels()]
+        problem.undefined = np.asarray(undefined)
 #CRUFT
 recall_best = load_best
 
@@ -161,7 +178,8 @@ def store_overwrite_query_gui(path):
     Use this in a call to :func:`make_store` from a graphical user interface.
     """
     import wx
-    msg_dlg = wx.MessageDialog(None, path + " already exists. Press 'yes' to overwrite, or 'No' to abort and restart with newpath", 'Overwrite Directory',
+    msg = path + " already exists. Press 'yes' to overwrite, or 'No' to abort and restart with newpath"
+    msg_dlg = wx.MessageDialog(None, msg, 'Overwrite Directory',
                                wx.YES_NO | wx.ICON_QUESTION)
     retCode = msg_dlg.ShowModal()
     msg_dlg.Destroy()
@@ -284,7 +302,7 @@ def initial_model(opts):
         np.random.seed(opts.seed)
 
     if opts.args:
-        problem = load_model(opts.args[0],opts.args[1:])
+        problem = load_model(opts.args[0], opts.args[1:])
         if opts.pars is not None:
             load_best(problem, opts.pars)
         if opts.simrandom:
@@ -371,7 +389,7 @@ def config_matplotlib(backend=None):
     if hasattr(sys, 'frozen'):
         if 'MPLCONFIGDIR' not in os.environ:
             raise RuntimeError(
-                "MPLCONFIGDIR should be set to e.g., %LOCALAPPDATA%\YourApp\mplconfig")
+                r"MPLCONFIGDIR should be set to e.g., %LOCALAPPDATA%\YourApp\mplconfig")
         if backend is None:
             backend = 'WXAgg'
 
@@ -390,14 +408,47 @@ def config_matplotlib(backend=None):
 
     matplotlib.interactive(False)
 
+    #configure the plot style
+    line_width = 1
+    pad = 2
+    font_family = 'Arial' if os.name == 'nt' else 'sans-serif'
+    font_size = 12
+    plot_style = {
+        'xtick.direction': 'in',
+        'ytick.direction': 'in',
+        'lines.linewidth': line_width,
+        'axes.linewidth': line_width,
+        'xtick.labelsize': font_size,
+        'ytick.labelsize': font_size,
+        'xtick.major.size': 5,
+        'ytick.major.size': 5,
+        'xtick.minor.size': 2.5,
+        'ytick.minor.size': 2.5,
+        'xtick.major.width': line_width,
+        'ytick.major.width': line_width,
+        'xtick.minor.width': line_width,
+        'ytick.minor.width': line_width,
+        'xtick.major.pad': pad,
+        'ytick.major.pad': pad,
+        'xtick.top': True,
+        'ytick.right': True,
+        'font.size': font_size,
+        'font.family': font_family,
+        'svg.fonttype': 'none',
+        'savefig.dpi': 100,
+    }
+    matplotlib.rcParams.update(plot_style)
 
 def beep():
     """
     Audio signal that fit is complete.
     """
     if sys.platform == "win32":
-        import winsound
-        winsound.MessageBeep(winsound.MB_ICONEXCLAMATION)
+        try:
+            import winsound
+            winsound.MessageBeep(winsound.MB_OK)
+        except Exception:
+            pass
     else:
         print("\a", file=sys.__stdout__)
 
@@ -410,13 +461,20 @@ def run_command(c):
 
 
 def setup_logging():
+    """Start logger"""
     import logging
     logging.basicConfig(level=logging.INFO)
 
 # from http://stackoverflow.com/questions/22373927/get-traceback-of-warnings
 # answered by mgab (2014-03-13)
 # edited by Gareth Rees (2015-11-28)
-def warn_with_traceback(message, category, filename, lineno, file=None, line=None):
+def warn_with_traceback(message, category, filename, lineno,
+                        file=None, line=None):
+    """
+    Alternate warning printer which shows a traceback with the warning.
+
+    To use, set *warnings.showwarning = warn_with_traceback*.
+    """
     traceback.print_stack()
     log = file if hasattr(file, 'write') else sys.stderr
     log.write(warnings.formatwarning(message, category, filename, lineno, line))
@@ -478,7 +536,7 @@ def main():
     if opts.mpi:
         MPIMapper.start_worker(problem)
         mapper = MPIMapper
-    elif opts.parallel or opts.worker:
+    elif opts.parallel != "" or opts.worker:
         if opts.transport == 'amqp':
             mapper = AMQPMapper
         elif opts.transport == 'mp':
@@ -497,9 +555,9 @@ def main():
         import time
         start_time = time.time()
         stop_time = start_time + float(opts.time)*3600
-        abort_test=lambda: time.time() >= stop_time
+        abort_test = lambda: time.time() >= stop_time
     else:
-        abort_test=lambda: False
+        abort_test = lambda: False
 
     fitdriver = FitDriver(
         opts.fit_config.selected_fitter, problem=problem, abort_test=abort_test,
@@ -512,25 +570,36 @@ def main():
         run_profiler(problem, steps=int(opts.steps))
     elif opts.chisq:
         if opts.cov:
-            print(problem.cov())
+            fitdriver.show_cov()
         print("chisq", problem.chisq_str())
     elif opts.preview:
         if opts.cov:
-            print(problem.cov())
+            fitdriver.show_cov()
         preview(problem, view=opts.view)
     elif opts.resynth > 0:
         resynth(fitdriver, problem, mapper, opts)
 
     elif opts.remote:
-
         # Check that problem runs before submitting it remotely
-        chisq = problem()
-        print("initial chisq:", chisq)
+        # TODO: this may fail if problem requires remote resources such as GPU
+        print("initial chisq:", problem.chisq_str())
         job = start_remote_fit(problem, opts,
                                queue=opts.queue, notify=opts.notify)
         print("remote job:", job['id'])
 
     else:
+        # Show command line arguments and initial model
+        print("#", " ".join(sys.argv))
+        problem.show()
+
+        # Check that there are parameters to be fitted.
+        if not len(problem.getp()):
+            print("\n!!! No parameters selected for fitting---abort !!!\n")
+            return
+
+        # Run the fit
+        if opts.resume == '-':
+            opts.resume = opts.store if os.path.exists(opts.store) else None
         if opts.resume:
             resume_path = os.path.join(opts.resume, problem.name)
         else:
@@ -538,30 +607,23 @@ def main():
 
         make_store(problem, opts, exists_handler=store_overwrite_query)
 
-        # Show command line arguments and initial model
-        print("#", " ".join(sys.argv))
-        problem.show()
         if opts.stepmon:
             fid = open(problem.output_path + '.log', 'w')
             fitdriver.monitors = [ConsoleMonitor(problem),
                                   StepMonitor(problem, fid, fields=['step', 'value'])]
 
         #import time; t0=time.clock()
-        fitdriver.mapper = mapper.start_mapper(problem, opts.args)
+        cpus = int(opts.parallel) if opts.parallel != "" else 0
+        fitdriver.mapper = mapper.start_mapper(problem, opts.args, cpus=cpus)
         best, fbest = fitdriver.fit(resume=resume_path)
         # print("time=%g"%(time.clock()-t0),file=sys.__stdout__)
         save_best(fitdriver, problem, best, view=opts.view)
         if opts.err or opts.cov:
             fitdriver.show_err()
         if opts.cov:
-            np.set_printoptions(linewidth=1000000)
-            print("=== Covariance matrix ===")
-            print(problem.cov())
-            print("=========================")
+            fitdriver.show_cov()
         if opts.entropy:
-            print("Calculating entropy...")
-            S, dS = fitdriver.entropy()
-            print("Entropy: %s bits" % format_uncertainty(S, dS))
+            fitdriver.show_entropy()
         mapper.stop_mapper(fitdriver.mapper)
         if not opts.batch and not opts.mpi and not opts.noshow:
             beep()
