@@ -5,19 +5,65 @@ Interface between the models and the fitters.
 These models can be bundled together into a :func:`FitProblem` and sent
 to :class:`bumps.fitters.FitDriver` for optimization and uncertainty
 analysis.
+
+
+Summary of problem attributes::
+
+    # Used by fitters
+    nllf(p: Optional[Vector]) -> float  # main calculation
+    bounds() -> Tuple(Vector, Vector)    # or equivalent sequence
+    setp(p: Vector) -> None
+    getp() -> Vector
+    residuals() -> Vector  # for LM, MPFit
+    parameter_residuals() -> Vector  # for LM, MPFit
+    constraints_nllf() -> float # for LM, MPFit;  constraint cost is spread across the individual residuals
+    randomize() -> None # for multistart
+    resynth_data() -> None  # for Monte Carlo resampling of maximum likelihood
+    restore_data() -> None # for Monte Carlo resampling of maximum likelihood
+    name: str  # DREAM uses this
+    chisq() -> float
+    chisq_str() -> str
+    labels() -> List[str]
+    summarize() -> str
+    show() -> None
+    load(input_path: str) -> None
+    save(output_path: str) -> None
+    plot(figfile: str, view: str) -> None
+
+    # Set/used by bumps.cli
+    model_reset() -> None # called by load_model
+    path: str  # set by load_model
+    name: str # set by load_model
+    title: str = filename # set by load_moel
+    options: List[str]  # from sys.argv[1:]
+    undefined:List[int]  # when loading a save .par file, these parameters weren't defined
+    store: str # set by make_store
+    output_path: str # set by make_store
+    simulate_data(noise: float) -> None # for --simulate in opts
+    cov() -> Matrix   # for --cov in opts
+
 """
+# Don't include print_function in imports; since the model coded is exec'd
+# in the __future__ context of this file, it would force the models to use the
+# new print function syntax.  load_problem() should be moved to its own file
+# to avoid this issue.
 from __future__ import division, with_statement
 
 __all__ = ['Fitness', 'FitProblem', 'load_problem',
            'BaseFitProblem', 'MultiFitProblem']
 
 import sys
+import os
+import traceback
+import logging
+import warnings
 
 import numpy as np
-from numpy import inf, isnan
+from numpy import inf, isnan, NaN
 
 from . import parameter, bounds as mbounds
 from .formatnum import format_uncertainty
+from . import util
 
 # Abstract base class
 class Fitness(object):
@@ -33,39 +79,39 @@ class Fitness(object):
         Model parameters are a hierarchical structure of lists and
         dictionaries.
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def update(self):
         """
         Called when parameters have been updated.  Any cached values will need
         to be cleared and the model reevaluated.
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def numpoints(self):
         """
         Return the number of data points.
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def nllf(self):
         """
         Return the negative log likelihood value of the current parameter set.
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def resynth_data(self):
         """
         Generate fake data based on uncertainties in the real data.  For
         Monte Carlo resynth-refit uncertainty analysis.  Bootstrapping?
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def restore_data(self):
         """
         Restore the original data in the model (after resynth).
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def residuals(self):
         """
@@ -73,7 +119,7 @@ class Fitness(object):
 
         Used for Levenburg-Marquardt, and for plotting.
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def save(self, basename):
         """
@@ -242,6 +288,7 @@ class BaseFitProblem(object):
         self.fitness.restore_data()
 
     def valid(self, pvec):
+        """Return true if the point is in the feasible region"""
         return all(v in p.bounds for p, v in zip(self._parameters, pvec))
 
     def setp(self, pvec):
@@ -277,6 +324,7 @@ class BaseFitProblem(object):
         return np.array([p.value for p in self._parameters], 'd')
 
     def bounds(self):
+        """Return the bounds fore each parameter a 2 x N array"""
         return np.array([p.bounds.limits for p in self._parameters], 'd').T
 
     def randomize(self, n=None):
@@ -323,10 +371,27 @@ class BaseFitProblem(object):
         """
         return [p.residual() for p in self.bounded]
 
+    @property
+    def has_residuals(self):
+        """
+        True if the underlying fitness function defines residuals.
+        """
+        return hasattr(self.fitness, 'residuals')
+
     def residuals(self):
-        """
+        r"""
         Return the model residuals.
+
+        If the model is defined by $y = f(x) + \epsilon$ for normally
+        distributed error in the measurement $y$ equal to
+        $\epsilon \sim N(0, \sigma^2)$, then residuals will be defined by
+        $R = (y - f(x))/\sigma$.  If the measurement uncertainty is not normal,
+        then the normal equivalent residuals should be defined so that the
+        Levenberg-Marquardt fit behaves reasonably, and the plot of
+        residuals gives an indication of which points are driving the fit.
         """
+        if not hasattr(self.fitness, 'residuals'):
+            raise NotImplemented("model does not define residuals")
         return self.fitness.residuals()
 
     def chisq(self):
@@ -343,19 +408,45 @@ class BaseFitProblem(object):
         return np.sum(self.residuals() ** 2) / self.dof
         # return 2*self.nllf()/self.dof
 
+    def chisq_str(self):
+        """
+        Return a string representing the chisq equivalent of the nllf.
+
+        If the model has strictly gaussian independent uncertainties then the
+        negative log likelihood function will return 0.5*sum(residuals**2),
+        which is 1/2*chisq.  Since we are printing normalized chisq, we
+        multiply the model nllf by 2/DOF before displaying the value.  This
+        is different from the problem nllf function, which includes the
+        cost of the prior parameters and the cost of the penalty constraints
+        in the total nllf.  The constraint value is displayed separately.
+        """
+        pparameter, pconstraints, pmodel = self._nllf_components()
+        chisq_norm, chisq_err = nllf_scale(self)
+        chisq = pmodel * chisq_norm
+        text = format_uncertainty(chisq, chisq_err)
+        constraints = pparameter + pconstraints
+        if constraints > 0.:
+            text += " constraints=%g" % constraints
+
+        return text
+
     def nllf(self, pvec=None):
         """
-        Compute the cost function for a new parameter set p.
+        compute the cost function for a new parameter set p.
 
-        Note that this is not simply the sum-squared residuals, but instead
-        is the negative log likelihood of seeing the data given the model plus
-        the negative log likelihood of seeing the model.  The individual
-        likelihoods are scaled by 1/max(P) so that normalization constants
-        can be ignored.
+        this is not simply the sum-squared residuals, but instead is the
+        negative log likelihood of seeing the data given the model parameters
+        plus the negative log likelihood of seeing the model parameters.  the
+        value is used for a likelihood ratio test so normalization constants
+        can be ignored.  there is an additional penalty value provided by
+        the model which can be used to implement inequality constraints.  any
+        penalty should be large enough that it is effectively excluded from
+        the parameter space returned from uncertainty analysis.
 
-        The model is not actually calculated if the parameter nllf plus the
+        the model is not actually calculated if the parameter nllf plus the
         constraint nllf are bigger than *soft_limit*, but instead it is
-        assigned a value of *penalty_nllf*.
+        assigned a value of *penalty_nllf*.  this will prevent expensive
+        models from spending time computing values in the unfeasible region.
         """
         if pvec is not None:
             if self.valid(pvec):
@@ -363,33 +454,35 @@ class BaseFitProblem(object):
             else:
                 return inf
 
+        pparameter, pconstraints, pmodel = self._nllf_components()
+        cost = pparameter + pconstraints + pmodel
+        # print(pvec, "cost=",pparameter,"+",pconstraints,"+",pmodel,"=",cost)
+        if isnan(cost):
+            # todo: make sure errors get back to the user
+            # print "point evaluates to nan"
+            # print parameter.summarize(self._parameters)
+            return inf
+        return cost
+
+    def _nllf_components(self):
         try:
-            if isnan(self.parameter_nllf()):
-                # TODO: make sure errors get back to the user
-                import logging
-                info = ["Parameter nllf is wrong"]
-                info += ["%s %g" %(p,p.nllf()) for p in self.bounded]
-                logging.error("\n  ".join(info))
             pparameter = self.parameter_nllf()
-            pconstraint = self.constraints_nllf()
+            if isnan(pparameter):
+                # TODO: make sure errors get back to the user
+                info = ["Parameter nllf is wrong"]
+                info += ["%s %g"%(p, p.nllf()) for p in self.bounded]
+                logging.error("\n  ".join(info))
+            pconstraints = self.constraints_nllf()
             pmodel = (self.model_nllf()
-                      if pparameter + pconstraint <= self.soft_limit
+                      if pparameter + pconstraints <= self.soft_limit
                       else self.penalty_nllf)
-            cost = pparameter + pconstraint + pmodel
+            return pparameter, pconstraints, pmodel
         except Exception:
             # TODO: make sure errors get back to the user
-            import traceback, logging
             info = (traceback.format_exc(),
                     parameter.summarize(self._parameters))
             logging.error("\n".join(info))
-            return inf
-        if isnan(cost):
-            # TODO: make sure errors get back to the user
-            # print "point evaluates to NaN"
-            # print parameter.summarize(self._parameters)
-            return inf
-        # print pvec, "cost",cost,"=",pparameter,"+",pconstraint,"+",pmodel
-        return cost
+            return NaN, NaN, NaN
 
     def __call__(self, pvec=None):
         """
@@ -402,22 +495,40 @@ class BaseFitProblem(object):
         """
         return 2 * self.nllf(pvec) / self.dof
 
-    def show(self):
-        print(parameter.format(self.model_parameters()))
+    def show(self, _subs={}):
+        """Print the available parameters to the console as a tree."""
+        print(parameter.format(self.model_parameters(), freevars=_subs))
         print("[chisq=%s, nllf=%g]" % (self.chisq_str(), self.nllf()))
         #print(self.summarize())
 
     def summarize(self):
+        """Return a table of current parameter values with range bars."""
         return parameter.summarize(self._parameters)
 
     def labels(self):
+        """Return the list of labels, one per fitted parameter."""
         return [p.name for p in self._parameters]
 
     def save(self, basename):
+        """
+        Save the problem state for the current parameter set.
+
+        The underlying Fitness object *save* method is called, if it exists,
+        so that theory values can be saved in a format suitable to the problem.
+
+        Uses *basename* as the base of any files that are created.
+        """
         if hasattr(self.fitness, 'save'):
             self.fitness.save(basename)
 
     def plot(self, p=None, fignum=None, figfile=None, view=None):
+        """
+        Plot the problem state for the current parameter set.
+
+        The underlying Fitness object *plot* method is called with *view*.
+        It should produce its plot on the current matplotlib figure.  This
+        method will add chisq to the plot and save it to a file.
+        """
         if not hasattr(self.fitness, 'plot'):
             return
 
@@ -433,12 +544,29 @@ class BaseFitProblem(object):
             pylab.savefig(figfile + "-model.png", format='png')
 
     def cov(self):
+        """
+        Return the covariance matrix as computed by numdifftools from the
+        Hessian matrix for the problem at the current parameter values.
+        """
+        # TODO: remove from model
+        warnings.warn("use cov and stderr from FitDriver, not problem.",
+                      DeprecationWarning)
+
         from . import lsqerror
         H = lsqerror.hessian(self)
         H, L = lsqerror.perturbed_hessian(H)
         return lsqerror.chol_cov(L)
 
     def stderr(self):
+        """
+        Return the 1-sigma uncertainty estimate for each parameter and the
+        correlation matrix *R* as computed from the covariance returned by
+        *cov*.
+        """
+        # TODO: remove from model
+        warnings.warn("use cov and stderr from FitDriver, not problem.",
+                      DeprecationWarning)
+
         from . import lsqerror
         c = self.cov()
         return lsqerror.stderr(c), lsqerror.corr(c)
@@ -452,21 +580,6 @@ class BaseFitProblem(object):
             self.soft_limit, self.constraints = state
         self.model_reset()
 
-    def chisq_str(self):
-        # TODO: remove unnecessary try-catch
-        try:
-            _, err = nllf_scale(self)
-            text = format_uncertainty(self.chisq(), err)
-            constraints = (self.parameter_nllf()
-                           + self.constraints_nllf())
-            if constraints > 0.:
-                text+= " constraints=%g"%constraints
-        except Exception:
-            # Otherwise indicate that chisq could not be calculated.
-            text = "--"
-
-        return text
-
 class MultiFitProblem(BaseFitProblem):
     """
     Weighted fits for multiple models.
@@ -555,9 +668,16 @@ class MultiFitProblem(BaseFitProblem):
         for f in self.models:
             f.restore_data()
 
+    @property
+    def has_residuals(self):
+        """
+        True if all underlying fitness functions define residuals.
+        """
+        return all(f.has_residuals for f in self.models)
+
     def residuals(self):
         resid = np.hstack([w * f.residuals()
-                              for w, f in zip(self.weights, self.models)])
+                           for w, f in zip(self.weights, self.models)])
         return resid
 
     def save(self, basename):
@@ -567,7 +687,8 @@ class MultiFitProblem(BaseFitProblem):
     def show(self):
         for i, f in enumerate(self.models):
             print("-- Model %d %s" % (i, f.name))
-            f.show()
+            subs = self.freevars.get_model(i) if self.freevars else {}
+            f.show(_subs=subs)
         print("[overall chisq=%s, nllf=%g]" % (self.chisq_str(), self.nllf()))
 
     def plot(self, p=None, fignum=1, figfile=None, view=None):
@@ -590,7 +711,7 @@ class MultiFitProblem(BaseFitProblem):
 
 
 # TODO: consider adding nllf_scale to FitProblem.
-ONE_SIGMA=0.68268949213708585
+ONE_SIGMA = 0.68268949213708585
 def nllf_scale(problem):
     r"""
     Return the scale factor for reporting the problem nllf as an approximate
@@ -626,7 +747,11 @@ def load_problem(filename, options=None):
 
     Raises ValueError if the script does not define problem.
     """
-    ctx = dict(__file__=filename, __name__="bumps_model")
+    # Allow relative imports from the bumps model
+    package = util.relative_import(filename)
+    module = os.path.splitext(os.path.basename(filename))[0]
+
+    ctx = dict(__file__=filename, __package__=package, __name__=module)
     old_argv = sys.argv
     sys.argv = [filename] + options if options else [filename]
     source = open(filename).read()
