@@ -62,6 +62,7 @@ class StepMonitor(monitor.Monitor):
             raise ValueError("invalid monitor field")
         self.fid = fid
         self.fields = fields
+        self.problem = problem
         self._pattern = "%%(%s)s\n" % (")s %(".join(fields))
         fid.write("# " + ' '.join(fields) + '\n')
 
@@ -182,12 +183,12 @@ class MultiStart(FitBase):
         f_best = np.inf
         x_best = self.problem.getp()
         for _ in range(max(starts, 1)):
-            logging.info("multistart round %d"%_)
+            logging.info("multistart round %d", _)
             x, fx = self.fitter.solve(monitors=monitors, mapper=mapper,
                                       **options)
             if fx < f_best:
                 x_best, f_best = x, fx
-                logging.info("multistart f(x),x: %s %s"%(str(fx),str(x_best)))
+                logging.info("multistart f(x),x: %s %s", str(fx), str(x_best))
             if reset:
                 self.problem.randomize()
             else:
@@ -208,7 +209,7 @@ class DEFit(FitBase):
     id = "de"
     settings = [('steps', 1000), ('pop', 10), ('CR', 0.9), ('F', 2.0),
                 ('ftol', 1e-8), ('xtol', 1e-6), #('stop', ''),
-                ]
+               ]
 
     def solve(self, monitors=None, abort_test=None, mapper=None, **options):
         if abort_test is None:
@@ -322,7 +323,7 @@ class BFGSFit(FitBase):
                              steptol=1e-12,
                              macheps=1e-8,
                              eta=1e-8,
-                             )
+                            )
         self.result = result
         #code = result['status']
         #from .quasinewton import STATUS
@@ -439,8 +440,8 @@ class PTFit(FitBase):
         self._update = MonitorRunner(problem=self.problem,
                                      monitors=monitors)
         t = np.logspace(np.log10(options['Tmin']),
-                           np.log10(options['Tmax']),
-                           options['nT'])
+                        np.log10(options['Tmax']),
+                        options['nT'])
         history = parallel_tempering(nllf=self.problem.nllf,
                                      p=self.problem.getp(),
                                      bounds=self.problem.bounds(),
@@ -496,6 +497,92 @@ class SimplexFit(FitBase):
                      population_points=x, population_values=fx)
         return True
 
+class MPFit(FitBase):
+    """
+    MPFit optimizer.
+    """
+    name = "MPFit"
+    id = "mp"
+    settings = [('steps', 200), ('ftol', 1e-10), ('xtol', 1e-10)]
+
+    def solve(self, monitors=None, abort_test=None, mapper=None, **options):
+        from .mpfit import mpfit
+        if abort_test is None:
+            abort_test = lambda: False
+        options = _fill_defaults(options, self.settings)
+        self._low, self._high = self.problem.bounds()
+        self._update = MonitorRunner(problem=self.problem,
+                                     monitors=monitors)
+        self._abort = abort_test
+        x0 = self.problem.getp()
+        parinfo = []
+        for low, high in zip(*self.problem.bounds()):
+            parinfo.append({
+                #'value': None,  # passed in by xall instead
+                #'fixed': False,  # everything is varying
+                'limited': (np.isfinite(low), np.isfinite(high)),
+                'limits': (low, high),
+                #'parname': '',  # could probably ask problem for this...
+                # From the code, default step size is sqrt(eps)*abs(value)
+                # or eps if value is 0.  This seems okay.  The other
+                # other alternative is to limit it by bounds.
+                #'step': 0,  # compute step automatically
+                #'mpside': 0,  # 1, -1 or 2 for right-, left- or 2-sided deriv
+                #'mpmaxstep': 0.,  # max step for this parameter
+                #'tied': '',  # parameter expressions tying fit parameters
+                #'mpprint': 1,  # print the parameter value when iterating
+            })
+
+        result = mpfit(
+            fcn=self._residuals,
+            xall=x0,
+            parinfo=parinfo,
+            autoderivative=True,
+            fastnorm=True,
+            #damp=0,  # no damping when damp=0
+            # Stopping conditions
+            ftol=options['ftol'],
+            xtol=options['xtol'],
+            #gtol=1e-100, # exclude gtol test
+            maxiter=options['steps'],
+            # Progress monitor
+            iterfunct=self._monitor,
+            nprint=1,  # call monitor each iteration
+            quiet=True,  # leave it to monitor to print any info
+            # Returns values
+            nocovar=True,  # use our own covar calculation for consistency
+        )
+
+        if result.status > 0:
+            x, fx = result.params, result.fnorm
+        else:
+            x, fx = None, None
+
+        return x, fx
+
+
+    def _monitor(self, fcn, p, k, fnorm,
+                 functkw=None, parinfo=None,
+                 quiet=0, dof=None, **extra):
+        self._update(k, p, fnorm)
+
+    def _residuals(self, p, fjac=None):
+        if self._abort():
+            return -1, None
+
+        self.problem.setp(p)
+        # treat prior probabilities on the parameters as additional
+        # measurements
+        residuals = np.hstack(
+            (self.problem.residuals().flat, self.problem.parameter_residuals()))
+        # Tally costs for broken constraints
+        extra_cost = self.problem.constraints_nllf()
+        # Spread the cost over the residuals.  Since we are smoothly increasing
+        # residuals as we leave the boundary, this should push us back into the
+        # boundary (within tolerance) during the lm fit.
+        residuals += np.sign(residuals) * (extra_cost / len(residuals))
+        return 0, residuals
+
 
 class LevenbergMarquardtFit(FitBase):
     """
@@ -543,7 +630,8 @@ class LevenbergMarquardtFit(FitBase):
         # covariance output and calculate it again ourselves.  Not ideal if
         # f is expensive, but it will be consistent with other optimizers.
         if x is not None:
-            self.problem.setp(x + self._stray_delta(x))
+            x += self._stray_delta(x)
+            self.problem.setp(x)
             fx = self.problem.nllf()
         else:
             fx = None
@@ -553,7 +641,8 @@ class LevenbergMarquardtFit(FitBase):
         # Force the fit point into the valid region
         stray = self._stray_delta(p)
         stray_cost = np.sum(stray**2)
-        if stray_cost > 0: stray_cost += 1e6
+        if stray_cost > 0:
+            stray_cost += 1e6
         self.problem.setp(p + stray)
         # treat prior probabilities on the parameters as additional
         # measurements
@@ -690,7 +779,7 @@ class DreamFit(FitBase):
             self.state.set_visible_vars(visible_vars)
         integer_vars = getattr(self.problem, 'integer_vars', None)
         if integer_vars is not None:
-            self.state.integer_vars(integer_vars)
+            self.state.set_integer_vars(integer_vars)
 
         x, fx = self.state.best()
 
@@ -729,9 +818,21 @@ class DreamFit(FitBase):
     #    return np.cov(self.state.draw().points[-1000:])
 
     def load(self, input_path):
-        from .dream.state import load_state
-        print("loading saved state (this might take awhile) ...")
-        self.state = load_state(input_path, report=100)
+        from .dream.state import load_state, path_contains_saved_state
+        if path_contains_saved_state(input_path):
+            print("loading saved state (this might take awhile) ...")
+            fn, labels = getattr(self.problem, 'derive_vars', (None, []))
+            self.state = load_state(input_path, report=100, derived_vars=len(labels))
+        elif input_path != problem.output_path:
+            # Only signal an error if --resume is different from --store.  If
+            # it is the same (e.g., because "--resume=-" was given on the
+            # command line) but missing, then silently skip the resume.  Note
+            # that we can't just check for the existence of the directory
+            # since we will already have created it before we get here.  We
+            # do want to signal an error if resume is different from store
+            # in order to catch typos on the command line instead of letting
+            # users quietly believe they are resuming.
+            raise RuntimeError("no mcmc saved at %r"%input_path)
 
     def save(self, output_path):
         self.state.save(output_path)
@@ -803,7 +904,7 @@ def _resampler(fitter, xinit, samples=100, restart=False, **options):
         # Restore the state of the problem
         fitter.problem.restore_data()
         fitter.problem.setp(xinit)
-        fitter.problem.model_update()
+        #fitter.problem.model_update()  # setp does model update
     return points
 
 
@@ -817,11 +918,15 @@ class FitDriver(object):
         self.monitors = monitors
         self.abort_test = abort_test
         self.mapper = mapper if mapper else lambda p: list(map(problem.nllf, p))
+        self.fitter = None
+        self.result = None
 
     def fit(self, resume=None):
 
-        if hasattr(self, '_cov'): del self._cov
-        if hasattr(self, '_stderr'): del self._stderr
+        if hasattr(self, '_cov'):
+            del self._cov
+        if hasattr(self, '_stderr'):
+            del self._stderr
         fitter = self.fitclass(self.problem)
         if resume:
             fitter.load(resume)
@@ -829,11 +934,11 @@ class FitDriver(object):
         if starts > 1:
             fitter = MultiStart(fitter)
         t0 = time.clock()
+        self.fitter = fitter
         x, fx = fitter.solve(monitors=self.monitors,
                              abort_test=self.abort_test,
                              mapper=self.mapper,
                              **self.options)
-        self.fitter = fitter
         self.time = time.clock() - t0
         self.result = x, fx
         if x is not None:
@@ -847,8 +952,13 @@ class FitDriver(object):
             from .dream import entropy
             return entropy.cov_entropy(self.cov()), 0
 
+    def chisq(self):
+        if not hasattr(self, '_chisq'):
+            self._chisq = self.problem.chisq()
+        return self._chisq
+
     def cov(self):
-        """
+        r"""
         Return an estimate of the covariance of the fit.
 
         Depending on the fitter and the problem, this may be computed from
@@ -864,16 +974,32 @@ class FitDriver(object):
         This should be faster and more accurate than the Hessian of nllf
         when you can use it.
         """
+        # Note: if fit() has not been run then self.fitter is None and in
+        # particular, self.fitter will not have a covariance matrix.  In
+        # this case, the code will fall through to computing the covariance
+        # matrix directly from the problem.  It will use the initial value
+        # stored in the problem parameters because results will also be None.
         if not hasattr(self, '_cov'):
             self._cov = None
             if hasattr(self.fitter, 'cov'):
                 self._cov = self.fitter.cov()
         if self._cov is None:
-            if hasattr(self.problem, 'residuals'):
-                J = lsqerror.jacobian(self.problem, self.result[0])
+            # Use Jacobian if residuals are available because it is faster
+            # to compute.  Otherwise punt and use Hessian.  The has_residuals
+            # attribute should be True if present.  It may be false if
+            # the problem defines a residuals method but doesn't really
+            # have residuals (e.g. to allow levenberg-marquardt to run even
+            # though it is not fitting a sum-square problem).
+            if hasattr(self.problem, 'has_residuals'):
+                has_residuals = self.problem.has_residuals
+            else:
+                has_residuals = hasattr(self.problem, 'residuals')
+            x = self.problem.getp() if self.result is None else self.result[0]
+            if has_residuals:
+                J = lsqerror.jacobian(self.problem, x)
                 self._cov = lsqerror.cov(J)
             else:
-                H = lsqerror.hessian(self.problem, self.result[0])
+                H = lsqerror.hessian(self.problem, x)
                 H, L = lsqerror.perturbed_hessian(H)
                 self._cov = lsqerror.chol_cov(L)
         return self._cov
@@ -886,15 +1012,32 @@ class FitDriver(object):
         existing evaluations within the fitter, or from numerical
         differentiation around the minimum.
         """
+        # Note: if fit() has not been run then self.fitter is None and in
+        # particular, self.fitter will not have a stderr method defined so
+        # it will compute stderr from covariance.
         if not hasattr(self, '_stderr'):
             self._stderr = None
             if hasattr(self.fitter, 'stderr'):
                 self._stderr = self.fitter.stderr()
         if self._stderr is None:
             # If no stderr from the fitter then compute it from the covariance
-            self._stderr = lsqerror.stderr(self.cov())
+            self._stderr = self.stderr_from_cov()
         return self._stderr
 
+    def stderr_from_cov(self):
+        """
+        Return an estimate of standard error of the fit from covariance matrix.
+
+        Unlike stderr, which uses the estimate from the underlying
+        fitter (DREAM uses the MCMC sample for this), *stderr_from_cov*
+        estimates the error from the diagonal of the covariance matrix.
+        Here, the covariance matrix may have been estimated by the fitter
+        instead of the Hessian.
+        """
+        if not hasattr(self, '_stderr_from_cov'):
+            self._stderr_from_cov = lsqerror.stderr(self.cov())
+        return self._stderr_from_cov
+
     def show(self):
         if hasattr(self.fitter, 'show'):
             self.fitter.show()
@@ -909,16 +1052,35 @@ class FitDriver(object):
         """
         # TODO: need cheaper uncertainty estimate
         # Note: error estimated from hessian diagonal is insufficient.
-        err = lsqerror.stderr(self.cov())
-        norm = np.sqrt(self.problem.chisq())
-        print("=== Uncertainty est. from curvature: par    dx           dx/sqrt(chisq) ===")
+        err = self.stderr_from_cov()
+        norm = np.sqrt(self.chisq())
+        print("=== Uncertainty from curvature:     name"
+              " value(unc.)    "
+              " value(unc./chi)) ===")
         for k, v, dv in zip(self.problem.labels(), self.problem.getp(), err):
-            print("%40s %-15s %-15s" %(k,
-                                       format_uncertainty(v, dv),
-                                       format_uncertainty(v, dv/norm),
-                                       ))
+            print("%40s %-15s %-15s" % (k,
+                  format_uncertainty(v, dv),
+                  format_uncertainty(v, dv/norm)))
         print("="*75)
 
+    def show_cov(self):
+        cov = self.cov()
+        maxn = 1000 # max array dims to print
+        cov_str = np.array2string(
+            cov,
+            max_line_width=20*maxn, threshold=maxn*maxn,
+            precision=6, #suppress_small=True,
+            separator=', ',
+        )
+        print("=== Covariance matrix ===")
+        print(cov_str)
+        print("=========================")
+
+    def show_entropy(self):
+        print("Calculating entropy...")
+        S, dS = fitdriver.entropy()
+        print("Entropy: %s bits" % format_uncertainty(S, dS))
+
     def save(self, output_path):
         # print "calling driver save"
         if hasattr(self.fitter, 'save'):
@@ -940,6 +1102,18 @@ class FitDriver(object):
         if hasattr(self.fitter, 'plot'):
             self.fitter.plot(output_path=output_path)
 
+    def _save_fit_cov(self, output_path):
+        model = getattr(self.problem, 'name', self.problem.__class__.__name__)
+        fitter = self.fitclass.id
+        cov = self.cov()
+        err = self.stderr_from_cov()
+        chisq = self.chisq()
+
+        state = {
+            'model': model,
+            'fitter': fitter,
+        }
+
 
 def _fill_defaults(options, settings):
     """
@@ -956,6 +1130,7 @@ FITTERS = [
     DreamFit,
     BFGSFit,
     LevenbergMarquardtFit,
+    MPFit,
     PSFit,
     PTFit,
     RLFit,
@@ -971,9 +1146,60 @@ FIT_ACTIVE_IDS = [
     DreamFit.id,
     BFGSFit.id,
     LevenbergMarquardtFit.id,
+    MPFit.id,
     ]
 
 FIT_DEFAULT_ID = SimplexFit.id
 
 assert FIT_DEFAULT_ID in FIT_ACTIVE_IDS
 assert all(f in FIT_AVAILABLE_IDS for f in FIT_ACTIVE_IDS)
+
+def fit(problem, method=FIT_DEFAULT_ID, verbose=False, **options):
+    """
+    Simplified fit interface.
+
+    Given a fit problem, the name of a fitter and the fitter options,
+    it will run the fit and return the best value and standard error of
+    the parameters.  If *verbose* is true, then the console monitor will
+    be enabled, showing progress through the fit and showing the parameter
+    standard error at the end of the fit, otherwise it is completely
+    silent.
+
+    Returns an *OptimizeResult* object containing "x" and "dx".  The
+    dream fitter also includes the "state" object, allowing for more
+    detailed uncertainty analysis.  Optimizer information such as the
+    stopping condition and the number of function evaluations are not
+    yet included.
+    """
+    from scipy.optimize import OptimizeResult
+
+    #verbose = True
+    if method not in FIT_AVAILABLE_IDS:
+        raise ValueError("unknown method %r not one of %s"
+                         % (method, ", ".join(sorted(FIT_ACTIVE_IDS))))
+    for fitclass in FITTERS:
+        if fitclass.id == method:
+            break
+    monitors = None if verbose else []  # default is step monitor
+    driver = FitDriver(
+        fitclass=fitclass, problem=problem, monitors=monitors,
+        **options)
+    x0 = problem.getp()
+    x, fx = driver.fit()
+    problem.setp(x)
+    dx = driver.stderr()
+    if verbose:
+        print("final chisq", problem.chisq_str())
+        driver.show_err()
+    result = OptimizeResult(
+        x=x, dx=driver.stderr(),
+        fun=fx,
+        success=True, status=0, message="successful termination",
+        #nit=0, # number of iterations
+        #nfev=0, # number of function evaluations
+        #njev, nhev # jacobian and hessian evaluations
+        #maxcv=0, # max constraint violation
+        )
+    if hasattr(driver.fitter, 'state'):
+        result.state = driver.fitter.state
+    return result
